import os
import types
import uvicorn
from typing import Literal
from fastapi import FastAPI
from core.config import Config
from colorama import Style, Fore
from core.logger import LoggerMiddleware
from fastapi.middleware.cors import CORSMiddleware


class ApiRunner:
    def __init__(
        self,
        app,
        appinfo: dict,
        database: dict,
        host: str = "127.0.0.1",
        port: int = 5000,
        log_level: Literal[
            "error", "warning", "info", "debug", "critical"
        ] = "critical",
        reload: bool = True,
    ):
        self.app = FastAPI()
        self.appinfo = appinfo
        self.database = database
        self.host = host
        self.port = port
        self.log_level = log_level
        self.reload = reload

        self.app.add_middleware(
            CORSMiddleware,
            allow_origins=Config.CORS_ORIGINS,
            allow_methods=Config.CORS_ALLOW_METHODS,
            allow_headers=Config.CORS_ALLOW_HEADERS,
            allow_credentials=Config.CORS_ALLOW_CREDENTIALS,
        )

        self.app.add_middleware(LoggerMiddleware, enabled=True)

    def AddMiddleware(self, middleware: list[type]):
        for ware in middleware:
            if types(ware) is type:
                self.app.add_middleware(ware)

    def __clear_screen(self):
        os_name = os.name

        if os_name == "nt":
            os.system("cls")
        elif os_name == "posix":
            os.system("clear")
        else:
            pass

    def run(self):
        self.__clear_screen()

        output = [
            "",
            f"{'Service Info'}",
            "",
            f"{self.appinfo['description']} Is Started At Port [ {self.port} ]",
            "",
            f"Service Name : [ {self.appinfo['name']} ]",
            "",
            f"Service Version : [ {self.appinfo['version']} ]",
            "",
            f"Service Author : [ {self.appinfo['author']} ]",
            "",
            f"{'Database Status : '}",
            "",
        ]

        max_length = max(len(value) for value in output)
        border_length = max_length + 4

        print(Style.BRIGHT + Fore.BLUE + "\n" + " +" * (border_length // 2 + 3))

        for value in output:
            color = Fore.BLUE
            DB_STATUS = "On Success"

            if "Database Status" in value:
                padding = (border_length - len(value + f"[ {DB_STATUS} ]")) // 2
                status_color = Fore.RED if "On Error" in DB_STATUS else Fore.GREEN
                print(
                    Style.BRIGHT
                    + color
                    + " + "
                    + " " * padding
                    + value
                    + status_color
                    + f"[ {DB_STATUS} ]"
                    + " " * (border_length - len(value + f"[ {DB_STATUS} ]") - padding)
                    + color
                    + " +"
                )
            else:
                padding = (border_length - len(value)) // 2
                print(
                    Style.BRIGHT
                    + color
                    + " + "
                    + " " * padding
                    + value
                    + " " * (border_length - len(value) - padding)
                    + " +"
                )

        print(Style.BRIGHT + Fore.BLUE + " +" * (border_length // 2 + 3))

        uvicorn.run(
            app=f"__main__:{self.app}",
            host=self.host,
            port=self.port,
            reload=self.reload,
            log_level=self.log_level,
        )
